@@ -8,10 +8,11 @@ package Device::BusPirate;
 use strict;
 use warnings;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use Carp;
 
+use Future::Mutex;
 use Future::Utils qw( repeat );
 use IO::Termios;
 use Time::HiRes qw( time );
@@ -213,15 +214,7 @@ sub enter_mutex
    my $self = shift;
    my ( $code ) = @_;
 
-   my $oldm = $self->{mutex} // $self->_new_future->done( $self );
-   $self->{mutex} = my $newm = $self->_new_future;
-
-   $oldm->then( $code )
-      ->then_with_f( sub {
-         my $f = shift;
-         $newm->done( $self );
-         $f
-      });
+   ( $self->{mutex} //= Future::Mutex->new )->enter( $code );
 }
 
 =head2 enter_mode
