@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::BusPirate::Mode );
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use Carp;
 
@@ -242,6 +242,21 @@ sub read
      };
 }
 
+sub _i2c_txn
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   $self->pirate->enter_mutex( sub {
+      $self->start_bit
+         ->then( $code )
+         ->followed_by( sub {
+            my $f = shift;
+            $self->stop_bit->then( sub { $f } );
+         });
+   });
+}
+
 =head2 send
 
    $i2c->send( $address, $bytes )->get
@@ -263,12 +278,8 @@ sub send
    $address >= 0 and $address < 0x80 or
       croak "Invalid I2C slave address";
 
-   $self->pirate->enter_mutex( sub {
-      $self->start_bit->then( sub {
-         $self->write( chr( $address << 1 | 0 ) . $bytes )
-      })->then( sub {
-         $self->stop_bit;
-      });
+   $self->_i2c_txn( sub {
+      $self->write( chr( $address << 1 | 0 ) . $bytes );
    });
 }
 
@@ -293,15 +304,9 @@ sub recv
    $address >= 0 and $address < 0x80 or
       croak "Invalid I2C slave address";
 
-   $self->pirate->enter_mutex( sub {
-      $self->start_bit->then( sub {
-         $self->write( chr( $address << 1 | 1 ) )
-      })->then( sub {
-         $self->read( $length )
-      })->then( sub {
-         my ( $bytes ) = @_;
-         $self->stop_bit->then_done( $bytes );
-      });
+   $self->_i2c_txn( sub {
+      $self->write( chr( $address << 1 | 1 ) )
+         ->then( sub { $self->read( $length ) });
    });
 }
 
@@ -327,19 +332,11 @@ sub send_then_recv
    $address >= 0 and $address < 0x80 or
       croak "Invalid I2C slave address";
 
-   $self->pirate->enter_mutex( sub {
-      $self->start_bit->then( sub {
-         $self->write( chr( $address << 1 | 0 ) . $bytes_out )
-      })->then( sub {
-         $self->start_bit # repeated START
-      })->then( sub {
-         $self->write( chr( $address << 1 | 1 ) )
-      })->then( sub {
-         $self->read( $read_len )
-      })->then( sub {
-         my ( $bytes_in ) = @_;
-         $self->stop_bit->then_done( $bytes_in );
-      });
+   $self->_i2c_txn( sub {
+      $self->write( chr( $address << 1 | 0 ) . $bytes_out )
+         ->then( sub { $self->start_bit }) # repeated START
+         ->then( sub { $self->write( chr( $address << 1 | 1 ) ) })
+         ->then( sub { $self->read( $read_len ) })
    });
 }
 
