@@ -129,6 +129,12 @@ sub new
    bless { mode => $mode }, $class;
 }
 
+sub sleep
+{
+   my $self = shift;
+   $self->{mode}->pirate->sleep( @_ );
+}
+
 sub power
 {
    my $self = shift;
@@ -302,6 +308,9 @@ sub configure
        speed => $self->_find_speed( $max_bitrate, @I2C_SPEEDS )
     ) if defined $max_bitrate;
 
+    # It's highly likely the user will want the pullups enabled here
+    push @f, $self->{mode}->pullup( 1 );
+
     Future->needs_all( @f );
 }
 
